@@ -3,7 +3,9 @@
 use std::collections::HashMap;
 use std::convert::{TryFrom, TryInto};
 
+use ergotree_ir::chain::address::AddressEncoderError;
 use ergotree_ir::chain::ergo_box::box_value::BoxValue;
+use ergotree_ir::chain::ergo_box::BoxTokens;
 use ergotree_ir::chain::ergo_box::ErgoBoxCandidate;
 use ergotree_ir::chain::ergo_box::NonMandatoryRegisterId;
 use ergotree_ir::chain::ergo_box::NonMandatoryRegisters;
@@ -11,7 +13,7 @@ use ergotree_ir::chain::ergo_box::NonMandatoryRegistersError;
 use ergotree_ir::chain::token::Token;
 use ergotree_ir::ergo_tree::ErgoTree;
 use ergotree_ir::mir::constant::Constant;
-use ergotree_ir::serialization::{SigmaSerializable, SigmaSerializationError};
+use ergotree_ir::serialization::{SigmaParsingError, SigmaSerializable, SigmaSerializationError};
 use thiserror::Error;
 
 /// ErgoBoxCandidate builder errors
@@ -41,6 +43,16 @@ pub enum ErgoBoxCandidateBuilderError {
     /// Serialization error
     #[error("serialization error: {0}")]
     SerializationError(#[from] SigmaSerializationError),
+    /// Parsing error
+    #[error("parsing error: {0}")]
+    ParsingError(#[from] SigmaParsingError),
+    /// When creating a Box, it can either have no tokens, or 1-255 tokens
+    #[error("Too many Tokens. The maximum number of Tokens in an Ergo Box is 255")]
+    TooManyTokensError,
+
+    /// AddressEncoder error
+    #[error("address encoder error: {0}")]
+    AddressEncoderError(#[from] AddressEncoderError),
 }
 
 /// Minted token info (id, amount, name, desc)
@@ -110,6 +122,9 @@ impl ErgoBoxCandidateBuilder {
     /// Calculate minimal box value for the current box serialized size(in bytes)
     pub fn calc_min_box_value(&self) -> Result<BoxValue, ErgoBoxCandidateBuilderError> {
         let box_size_bytes = self.calc_box_size_bytes()?;
+
+        // Won't be overflowing an i64, so unwrap is safe.
+        #[allow(clippy::unwrap_used)]
         Ok(
             BoxValue::try_from(box_size_bytes as i64 * BoxValue::MIN_VALUE_PER_BOX_BYTE as i64)
                 .unwrap(),
@@ -215,6 +230,14 @@ impl ErgoBoxCandidateBuilder {
             );
         }
         let regs = NonMandatoryRegisters::new(additional_registers)?;
+        let tokens = if tokens.is_empty() {
+            None
+        } else {
+            Some(
+                BoxTokens::from_vec(tokens)
+                    .map_err(|_| ErgoBoxCandidateBuilderError::TooManyTokensError)?,
+            )
+        };
         let b = ErgoBoxCandidate {
             value: self.value,
             ergo_tree: self.ergo_tree.clone(),
@@ -223,6 +246,9 @@ impl ErgoBoxCandidateBuilder {
             creation_height: self.creation_height,
         };
         let box_size_bytes = b.sigma_serialize_bytes()?.len();
+
+        // Won't be overflowing an i64, so unwrap is safe.
+        #[allow(clippy::unwrap_used)]
         let min_box_value: BoxValue = (box_size_bytes as i64 * self.min_value_per_byte as i64)
             .try_into()
             .unwrap();
@@ -243,6 +269,7 @@ impl ErgoBoxCandidateBuilder {
 }
 
 #[cfg(test)]
+#[allow(clippy::unwrap_used)]
 mod tests {
 
     use ergotree_ir::base16_str::Base16Str;
@@ -345,7 +372,7 @@ mod tests {
             ErgoBoxCandidateBuilder::new(out_box_value, force_any_val::<ErgoTree>(), 0);
         box_builder.mint_token(token_pair.clone(), token_name, token_desc, token_num_dec);
         let out_box = box_builder.build().unwrap();
-        assert_eq!(out_box.tokens.get(0).unwrap(), &token_pair);
+        assert_eq!(out_box.tokens.unwrap().get(0).unwrap(), &token_pair);
         // test registers are encoded according to https://github.com/ergoplatform/eips/blob/master/eip-0004.md
         assert_eq!(
             out_box
@@ -434,6 +461,6 @@ mod tests {
             ErgoBoxCandidateBuilder::new(out_box_value, force_any_val::<ErgoTree>(), 0);
         box_builder.add_token(token.clone());
         let out_box = box_builder.build().unwrap();
-        assert_eq!(out_box.tokens.first().unwrap(), &token);
+        assert_eq!(out_box.tokens.unwrap().first(), &token);
     }
 }
