@@ -8,8 +8,8 @@ use super::prover_result::ProverResult;
 use super::DataInput;
 use super::Transaction;
 use super::TxIoVec;
+use ergo_chain_types::blake2b256_hash;
 use ergotree_interpreter::sigma_protocol::prover::ProofBytes;
-use ergotree_ir::chain::digest32::blake2b256_hash;
 use ergotree_ir::chain::ergo_box::ErgoBoxCandidate;
 use ergotree_ir::chain::token::TokenId;
 use ergotree_ir::chain::tx_id::TxId;
@@ -138,6 +138,7 @@ impl TryFrom<json::transaction::UnsignedTransactionJson> for UnsignedTransaction
 }
 
 #[cfg(test)]
+#[allow(clippy::unwrap_used, clippy::panic)]
 pub mod tests {
     use super::*;
 
